import { SAnimatedView } from './SAnimatedView';
import { Animated } from 'react-native';
import { SAnimatedProperty } from './SAnimatedProperty';
export declare type SAnimatedPropertyName = 'translateX' | 'translateY' | 'opacity' | 'scale' | 'ios-width' | 'ios-height';
export declare type SAnimatedPropertyAnimator = (name: string, property: SAnimatedPropertyName, from: number, to: number) => void;
export declare type SAnimatedEasing = 'linear' | 'material' | {
    bezier: number[];
};
export interface SAnimatedTimingConfig {
    property: SAnimatedPropertyName;
    from: number;
    to: number;
    duration?: number;
    delay?: number;
    optional?: boolean;
    easing?: SAnimatedEasing;
}
export interface SAnimatedCircularConfig {
    from: number;
    to: number;
    duration?: number;
    centerX?: number;
    centerY?: number;
}
export interface SAnimatedSpringConfig {
    property: SAnimatedPropertyName;
    from: number;
    to: number;
    duration?: number;
    delay?: number;
    optional?: boolean;
    velocity?: number;
}
export interface SAnimatedDynamic {
    translateX?: Animated.AnimatedInterpolation;
    translateY?: Animated.AnimatedInterpolation;
    opacity?: Animated.AnimatedInterpolation;
}
declare class SAnimatedImpl {
    View: typeof SAnimatedView;
    private _inTransaction;
    private _pendingAnimations;
    private _pendingSetters;
    private _transactionDuration;
    private _callbacks;
    private _propertyAnimator?;
    private _dirtyProperties;
    private _transactionKey?;
    private _knownComponents;
    private _subscriptions;
    private _dynamic;
    constructor();
    onPropertyChanged: (property: SAnimatedProperty, oldValue: number) => void;
    setValue: (name: string, property: SAnimatedPropertyName, value: number) => void;
    getDynamic(name: string): SAnimatedDynamic;
    startDynamic(name: string, property: SAnimatedPropertyName, value: Animated.AnimatedInterpolation): void;
    stopDynamic(name: string, property: SAnimatedPropertyName): void;
    readonly isInTransaction: boolean;
    readonly isInAnimatedTransaction: boolean;
    beginTransaction: () => void;
    setDuration: (duration: number) => void;
    setPropertyAnimator: (animator: SAnimatedPropertyAnimator) => void;
    setDefaultPropertyAnimator: () => void;
    addTransactionCallback: (callback: () => void) => void;
    commitTransaction: (callback?: (() => void) | undefined) => void;
    timing: (name: string, animation: SAnimatedTimingConfig) => void;
    spring: (name: string, animation: SAnimatedSpringConfig) => void;
    circular: (name: string, animation: SAnimatedCircularConfig) => void;
    private _postAnimations;
    onViewMounted: (name: string, handler: (src: SAnimatedDynamic) => void) => () => void;
}
export declare const SAnimated: SAnimatedImpl;
export {};
